@@ -15,7 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-__version__ = version = '0.2.19'
+__version__ = version = '0.2.20'
 
 release = 'kryptonite'
 
@@ -36,14 +36,16 @@ from lettuce.terrain import after
 from lettuce.terrain import before
 from lettuce.terrain import world
 
-from lettuce.decorators import step
+from lettuce.decorators import step, steps
 from lettuce.registry import call_hook
 from lettuce.registry import STEP_REGISTRY
 from lettuce.registry import CALLBACK_REGISTRY
 from lettuce.exceptions import StepLoadingError
 from lettuce.plugins import (
     xunit_output,
-    autopdb
+    subunit_output,
+    autopdb,
+    smtp_mail_queue,
 )
 from lettuce import fs
 from lettuce import exceptions
@@ -59,6 +61,7 @@ __all__ = [
     'after',
     'before',
     'step',
+    'steps',
     'world',
     'STEP_REGISTRY',
     'CALLBACK_REGISTRY',
@@ -86,8 +89,11 @@ class Runner(object):
     features and step definitions on there.
     """
     def __init__(self, base_path, scenarios=None, verbosity=0, random=False,
-                 enable_xunit=False, xunit_filename=None, tags=None,
-                 failfast=False, auto_pdb=False):
+                 enable_xunit=False, xunit_filename=None,
+                 enable_subunit=False, subunit_filename=None,
+                 tags=None, failfast=False, auto_pdb=False,
+                 smtp_queue=None):
+
         """ lettuce.Runner will try to find a terrain.py file and
         import it from within `base_path`
         """
@@ -124,6 +130,11 @@ class Runner(object):
 
         if enable_xunit:
             xunit_output.enable(filename=xunit_filename)
+        if smtp_queue:
+            smtp_mail_queue.enable()
+
+        if enable_subunit:
+            subunit_output.enable(filename=subunit_filename)
 
         reload(output)
 
@@ -133,12 +144,6 @@ class Runner(object):
         """ Find and load step definitions, and them find and load
         features under `base_path` specified on constructor
         """
-        try:
-            self.loader.find_and_load_step_definitions()
-        except StepLoadingError, e:
-            print "Error loading step definitions:\n", e
-            return
-
         results = []
         if self.single_feature:
             features_files = [self.single_feature]
@@ -151,6 +156,15 @@ class Runner(object):
             self.output.print_no_features_found(self.loader.base_dir)
             return
 
+        # only load steps if we've located some features.
+        # this prevents stupid bugs when loading django modules
+        # that we don't even want to test.
+        try:
+            self.loader.find_and_load_step_definitions()
+        except StepLoadingError, e:
+            print "Error loading step definitions:\n", e
+            return
+
         call_hook('before', 'all')
 
         failed = False
@@ -180,6 +194,7 @@ class Runner(object):
 
         finally:
             total = TotalResult(results)
+            total.output_format()
             call_hook('after', 'all', total)
 
             if failed:
